@@ -22,7 +22,10 @@ import functools
 
 __all__ = ['require',
            'export',
-           'extend']
+           'extend',
+           'oneshot',
+           'singleton']
+
 
 
 class Export(object):
@@ -36,11 +39,14 @@ class Export(object):
         extends are called after instance creation to manipulate the instance.
     """
 
-    def __init__(self, factory):
+    def __init__(self,
+                 factory,
+                 scope):
         self.__factory = factory
         self.__extenders = []
 
-        self.__instance = None
+        self.__proxy = scope(self.create)
+
 
     def extend(self, extender):
         """ Extends this export.
@@ -53,36 +59,43 @@ class Export(object):
             with the returned value.
         """
 
-        assert self.__instance is None
-
         self.__extenders.append(extender)
 
-    @property
-    def instance(self):
-        """ Returns the instance.
+        return extender
 
-            If no instance exists, it is created as described above.
+
+    def __call__(self):
+        """ Returns an instance of this export.
         """
 
-        # Create an instance none is available
-        if self.__instance is None:
+        return self.__proxy()
 
-            # Lazily create the instance by calling the factory
-            instance = self.__factory()
 
-            # Call all extenders for this export to update or replace the instance
-            for extender in self.__extenders:
-                extended = extender(instance)
+    def create(self):
+        """ Creates a new instance.
 
-                # If the extender returned a not-none value, the instance is
-                # replaced with the returned value
-                if extended is not None:
-                    instance = extended
+            The instance is created by calling the exported factory and
+            applying all extenders in the order of registration.
+        """
 
-            # Remember the instance for further requisition
-            self.__instance = instance
+        # Lazily create the instance by calling the factory
+        instance = self.__factory()
+
+        # Call all extenders for this export to update or replace the instance
+        for extender in self.__extenders:
+            # If the extender is an export, instantiate it
+            if isinstance(extender, Export):
+                extender = extender()
+
+            extended = extender(instance)
+
+            # If the extender returned a not-none value, the instance is
+            # replaced with the returned value
+            if extended is not None:
+                instance = extended
+
+        return instance
 
-        return self.__instance
 
     @staticmethod
     def load(requirement):
@@ -113,6 +126,35 @@ class Export(object):
         return export
 
 
+
+def oneshot(export):
+    """ Scope 'oneshot'.
+
+        The scope does not apply any caching or reusing of exported objects.
+        Each request for en export wil recreate the exported object.
+    """
+    return export
+
+
+
+def singleton(export):
+    """ Scope 'singleton'.
+
+        The singleton scope ensures, that only one single instance exists
+        during application life-time. The instance is created on first request
+        and cached for later use.
+    """
+    def wrapper():
+        if wrapper.instance is None:
+            wrapper.instance = export()
+
+        return wrapper.instance
+    wrapper.instance = None
+
+    return wrapper
+
+
+
 def require(requirement=None,
             **requirements):
     """ Factory for a function decorator or property descriptor.
@@ -131,7 +173,8 @@ def require(requirement=None,
         # Load the export specified in the requirement
         export = Export.load(requirement)
 
-        return property(lambda inst: export.instance)
+        # Build a property accessor returning the required instance
+        return property(lambda inst: export())
 
     elif requirements:
         # Load all exports as specified in the requirements
@@ -139,13 +182,12 @@ def require(requirement=None,
                    for name
                    in requirements}
 
-
         def wrapper(func):
             @functools.wraps(func)
             def wrapped(*args, **kwargs):
                 # Populate the keyword arguments dicts passed to the wrapped
                 # function with the required instances
-                kwargs.update({name: exports[name].instance
+                kwargs.update({name: exports[name]()
                                for name
                                in exports
                                if name not in kwargs})
@@ -162,7 +204,9 @@ def require(requirement=None,
         return lambda func: func
 
 
-def export(**requirements):
+
+def export(scope=singleton,
+           **requirements):
     """ Decorator to export a factory.
 
         Requirements for the factory can be specified. These requirements are
@@ -170,19 +214,25 @@ def export(**requirements):
     """
 
     def wrapper(func):
-        return Export(factory = require(**requirements)(func))
+        return Export(factory=require(**requirements)(func),
+                      scope=scope)
 
     return wrapper
 
 
-def extend(requirement):
+
+def extend(requirement,
+           **requirements):
     """ Decorator to extend a export.
 
         The decorated function is used to extend the export specified by the given
         name.
+
+        Requirements for the factory can be specified. These requirements are
+        passed to the wrapped function as specified by the `require` decorator.
     """
 
     def wrapper(func):
-        Export.load(requirement).extend(extender = func)
+        return Export.load(requirement).extend(extender=require(**requirements)(func))
 
     return wrapper
